#############################################
#### CSDE Workshop Series		              ###
#### Exploratory Spatial Data Analysis    ###
#############################################

#Prepared by Chris Fowler December 28th 2010
#The purpose of this file is to reproduce in R the techniques
#employed using ArcGIS within the ESDA Workshop

#This lab contains more detailed explanations of some of the functions and steps
#in order to support people working with this script or portions theoreof in their
#own work. Tasks that are not strictly necessary for following the lab are commented
#out.

## Install packages
install.packages("maps")
install.packages("maptools")
install.packages("sp")
install.packages("rgdal")
install.packages("RColorBrewer")
install.packages("plotrix")
install.packages("classInt")
install.packages("spdep")
install.packages("spatstat")

## Load packages
library(maps)         
library(maptools)     
library(sp)           
library(rgdal) 
library(RColorBrewer) 
library(plotrix)
library(classInt)     
library(spdep)    
library(spatstat)    

#This file assumes that the working directory has been set such that a folder /Data
#exists with the shapefiles "Financial Points" and "Seattle Blockgroups" inside it.   

# setwd("[your directory here]")
s

################################################################################
#If you have already gone through the lab in ArcGIS you may be surprised to
#learn how much is being taken care of for you behind the scenes. We encounter
#this in our very first line of R code when we go to read a shapefile into R
#
#The easiest way to do this is to simply read the shape in using readShapePoly
#from the maptools library
#seattle<-readShapePoly("Data/Seattle_blockgroups.shp")
#plot(seattle) #looks good but....
#
#str(seattle)   #...check out the very last line--this function doesn't read
               #the .prj file that contains the projection information for our
               #data. Without this any spatial analysis we do will be compromised
################################################################################

#For the purposes of this lab I have the correct projection information
projection<-"+proj=lcc +lat_1=47.5 +lat_2=48.73333333333333 +lat_0=47 +lon_0=-120.8333333333333 +x_0=500000.0000000001 +y_0=0 +ellps=GRS80 +units=us-ft +no_defs"

#now we can include this projection info as we read in the file and we are good
#to go
seattle<-readShapePoly("Data/Seattle_blockgroups.shp", proj4string=CRS(projection))
plot(seattle)
################################################################################
# If you are using this lab as a template for running your own analysis things
# may be somewhat more complex. Likely, you don't know the CRS form for your 
# projection. Often the easiest way to access this infor is using the web site
# spatialreference.org, but as of the creation of this document that site is
# unavailable. Another way to work through the problem is to use a different
# tool to read in your data. In this case we will use rgdal.
#         
#seattle<-readOGR(dsn="[directory in which all shapefile components are located]", layer="[layer name without .shp] ")
#plot(seattle)
#proj4string(seattle)
#
# this returns the same result (and is in fact how I got the CRS string above
# but I have had lots of trouble with rgdal and didn't want to get bogged down
# unnecessarily. If rgdal isn't working for you try installing the GDAL libraries
# using the downloads from http://fwtools.maptools.org/   BEFORE you install the
# rgdal library on your computer. I, as an R novice, had to move to a machine
# where I had admin priviledges, completely uninstall R, install FWTools, reinstall
# the latest R version, then download rgdal. Hope you find it easier...
################################################################################

### Having opened our shapefile we now want to access the data associated with it
# Converting the map to a data frame will automatically separate the complex spatial
# data and leave us with just the attribute values. Later on we will see how to join
# this attribute data back to the shapefile for output.

data<-as.data.frame(seattle)
str(data)

#look at our main variable as a histogram
hist(data$PctPov, breaks=20)

#Now draw our map again, but emphasize those tracts where PctPov is equal to zero
zeroes <- ifelse(seattle$PctPov ==0,"black","grey")
par(mar=rep(0,4))
plot(seattle,col=zeroes,border="black")
legend(x="left",cex=.7,fill=c("black","grey"),bty="n",legend=c("Zero Poverty","All Other"),title="Tracts with Zero Poverty",ncol=2)
dev.off() 

#Repeat for Poverty higher than 50%
highs <- ifelse(seattle$PctPov >=50,"black","grey")
par(mar=rep(0,4))
plot(seattle,col=highs,border="dark grey")
legend(x="left",cex=.7,fill=c("black","grey"),bty="n",legend=c("Poverty Greater than 50%","All Other"),title="Tracts with High (>50%) Poverty",ncol=2)
dev.off() 

#Our next trick will be to locate the polygon that represents the shortest total
#travel distance for all polygons within our data set. This is called "Central
#Feature" within ArcGIS and I have not been able to locate an equivalent function
#in an R package, so I have written one here that seems to do the trick. I am not
#an expert programmer, and even as I went through this lab I found ways I could have
#improved this function (namely by doing more of the operations in matrix form as
#opposed to loop form).

########
#for those not familiar with functions in R, to make our own function we assign
# it a name and then document the inputs the function will receive. We then specify
# the actions the function should take and return whatever values we expect. 
# Running the code below will 'teach' R this function for this session so that we
# can then use it, below we will actually call the function on our data
########

#Begin function CENTRAL FEATURE #      
central.feature<-function(points, weights=1){
 temp<-points
 if(length(weights)!=length(points[,1])){
    wts<-c(rep(1,length(temp[,1])))
 }else         {
    wts<-weights
 }   
 best_so_far<- -999
 distance<--999
 #start with a for loop that checks each point to see if it is lower than the
 #current best
  for (i in 1:length(temp[,1])){
          distance_to_i <- 0
          #calc the total distance from i to all of the other points in the set
          i.x<-temp[i,1]
          i.y<-temp[i,2]
          for (j in 1:length(temp[,1])){
              j.x<-temp[j,1]
              j.y<-temp[j,2]
              dif.x<-i.x-j.x
              dif.y<-i.y-j.y
              dist.xy<- sqrt(dif.x^2 + dif.y^2) * wts[j]
              distance_to_i<-distance_to_i + dist.xy
          }
          # if this is the first point we are testing then test here for baseline
          if(best_so_far == -999){
              best_so_far = i
              distance<- distance_to_i
          }
          # if the total distance to all points is lower than for any previous
          # point then this is our winner so far
          if(distance > distance_to_i){
              best_so_far = i
              distance<- distance_to_i
          }
  }
 return (temp[best_so_far,])
}
# End function CENTRAL FEATURE #


# Having programmed the function in, we can now use it.

#for this to work we need to use our the centroids of our polygons
#the coordinates function is designed to return exactly this information
coords<-coordinates(seattle)      

cf<-central.feature(coords)   #run our homemade function on the centroids

cf2<-central.feature(coords,weights=data$TotalPop)#run it again, this time weighting
                                                  #observations by population


# we will open a graphics window
par(mar=rep(0,4))

#plot our base layer
plot(seattle)

#insert periods to show the location of all of our blockgroup centroids
points(coords, pch=".")

points(x=cf[1],y=cf[2], pch=19, col="red", bg="red")    # plot our unweighted central feature
points(x=cf2[1],y=cf[2],pch=2,col="blue")               # plot our population weighted central feature

################################################################################
#Our homemade function nearly replicates the result in ArcGIS when using the
#Central Feature tool. ArcGIS locates the central feature in the blockgroup (drawn
#in two pieces)immediately to the east of the point found by our function. 
#Why the difference? 
#R computes the centroids slightly differently than ArcGIS, particularly
#with multipart polygons (as is the case with the blockgroup chosen by ArcGIS).
#In R the centroid for this polygon is located squarely inside the northern half
#of the blockgroup with the result that the centroid shown in red is selected.
#In contrast, ArcGIS locates the centroid of the two part polygon to the south 
#(actually in the boundary of the polygon R selected)so it manages to edge out 
#our red circle. I suspect the discrepancy could be resolved by altering the 
#parameters of the function that reads the shapefile in but am not entirely 
#sure for now how to accomplish this. When weights are used (as in cf2) both
# R and ArcGIS arrive at the same answer  (that is, ArcGIS moves its calculation
# to the same point as we calculated here in R)
################################################################################

dev.off() 

#The next portion of the lab looks at the mean center. Again this is most simply
#achieved by writing our own function--this one could be even simpler than the 
#function above except this time we are going to code the function to allow 
#the calculation of different cases in our data. We could go back and update
#our central feature function to incorporate this change without too much
#difficulty if we wanted to.

# Begin function MEAN CENTER #
mean.center<-function(points, weights=1, cases=1){
 temp<-points
 if(length(weights)!=length(points[,1])){
    wts<-c(rep(1,length(temp[,1])))
 }else         {
    wts<-weights
 }
 if(length(cases)!=length(points[,1])){
    case<-c(rep(1,length(temp[,1])))
 }else         {
    case<-cases
 }
 #we create a matrix to store our results. We will need one point for each
 #unique case type
 result<-data.frame(unique(case))
 result[,2:6]<-as.numeric(0)
 
 #now cycle through all of the points and add coordinates together. Also keep 
 #track of the number of instances of each case
 for(i in 1:length(temp[,1])){
    row<-which(result[,1]==case[i])
    result[row,2]=result[row,2]+(wts[i]*temp[i,1])
    result[row,3]=result[row,3]+(wts[i]*temp[i,2])
    result[row,4]=result[row,4]+ wts[i]
 }
 #now use the sums and number of instances to generate an average coordinate for
 #each unique type
 for(j in 1:length(result[,1])){
    result[j,5]<-result[j,2]/result[j,4]
    result[j,6]<-result[j,3]/result[j,4]
 }
 output<-result[,c(1,5,6)]
 return (output)
}
# End function MEAN CENTER #


#we will try this on a new data set showing the location of different types of
#financial institution within the city of Seattle.

#read in the shapefile. Note we use the same projection string as for the blockgroups
finance<-readShapePoints("Data/Financial_Points.shp", proj4string=CRS(projection))
data2<-as.data.frame(finance)
str(data2)   #note that column three is a variable that distinguishes among 5 types
             #of financial services

mc<-mean.center(data2[,4:5],cases=data2[,3]) #find the mean center for each type
mc  #look at the coordinates returned by our function

################################################################################
# Pause here to think a little bit about a small step towards drawing better maps in R. 
# R, despite its flexible graphics engine is not the ideal tool for practicing good
# cartography. For all its faults, ArcGIS is very good in this capacitya, and it
# is likely worth writing out your shapefile and results and making presentation
# materials there. However, that may not be feasible for all users, and R certainly
# has the capacity to turn out beautful maps if you can take the time to tweak all the
# features. In the short run, we can at least draw our maps with good color choices
#
# Color Brewer is a project out of the University of Pennsylavania that seeks to
# identify color combinations that are most readily interpreted by the human eye.
# Color Brewer choices are also available in ArcGIS, but they are perhaps even more
# accessible here through the library RColorBrewer.
#
#invoke the pallettes present in ColorBrewer
par(mar=c(0,3,0,0),cex=.6)
display.brewer.all(n=5)
dev.off()
#
# we can choose one of the above palettes and use it consistantly throughout the
# rest of the lab
#
#set the palette up based on the number of unique cases  note the maximum here is 8
mypalette <- brewer.pal(n=length(mc[,1]),"Accent")
################################################################################

#Map the results of our mean center function
par(mar=rep(0,4))
plot(seattle)
points(x=data2[,4],y=data2[,5], cex=.5, pch=19, col="grey") #map our data points
points(x=mc[,2],y=mc[,3], pch=20, col=mypalette, cex=4)    #map our mean centers
legend(x="left",cex=1, fill=mypalette,bty="n",legend=mc[,1],
title="Mean Centers")                                      #a basic legend
dev.off() 


#next we need a function to define the standard distance for our points
#the sdd is the radius of a circle (and its center point--the mean center)
#that is the minimum circle to enclose 68%  of our observations
#Again, I couldn't find a suitable pre-packaged function, so I wrote my own.

#Begin function STANDARD DISTANCE
# we will take as our input a matrix of coordinates (2 columns), and optionally
# a vector of weights (should be of the same length as there are pairs of coordinates
# we can also include a vector of the same length that defines each record's membership
# in a particular class.
standard.distance<-function (points, weights=1, cases=1)        {
  temp<-points
  if(length(weights)!=length(points[,1])){
    wts<-c(rep(1,length(temp[,1])))
  }else         {
    wts<-weights
  }
  if(length(cases)!=length(points[,1])){
    case<-c(rep(1,length(temp[,1])))
  }else         {
    case<-cases
  }
  #we create a matrix to store our results. We will need one point for each
  #unique case type
  result<-data.frame(unique(case))
  result[,2:4]<-as.numeric(0)
  #first calculate the mean centers for each case and place in the results column 2
        for(i in 1:length(result[,1])){
          each.case<-temp[which(cases==result[i,1]),]
          temp.wts<-wts[which(cases==result[i,1])]
          wt.x <- each.case[, 1] * temp.wts
          wt.y <- each.case[, 2] * temp.wts
          WMC.x <- c(sum(wt.x)/sum(temp.wts))
          WMC.y <- c(sum(wt.y)/sum(temp.wts))
          result[i,2] <- WMC.x
          result[i,3] <- WMC.y
        }
  #Now we need to calculate the distance to this mean center again by cases
         for(i in 1:length(result[,1])){
          each.case<-temp[which(cases==result[i,1]),]
          temp.wts<-wts[which(cases==result[i,1])]
          temp.dist<-as.data.frame(temp.wts)
          temp.dist$dist.x<-0
          temp.dist$dist.y<-0
            for(j in 1:length(each.case[,1])){
              temp.dist$dist.x[j]<-(result[i,2]-each.case[j,1])^2
              temp.dist$dist.y[j]<-(result[i,3]-each.case[j,2])^2
            }
            temp.dist$total<-sqrt(temp.dist$dist.x+temp.dist$dist.y)
            sum.weights<-sum(temp.wts)
            cutoff<-.68*sum.weights
            case.result<-0
            weights.so.far<-0
            o<-temp.dist[order(temp.dist[,4]),]
            counter<-1
            while(weights.so.far < cutoff){
              case.result<-o[counter,4]
              counter <-counter +1
              weights.so.far <- weights.so.far + o[counter,1]
            }
          result[i,4]<-case.result
        }
  return(result)        
}
#End Function STANDARD DISTANCE #

#Now that we have our function we can use it to collect mean centers and associated
#circle radii for our data on financial points
sd<-standard.distance(points=data2[,4:5],cases=data2[,3])

#now we need to draw our result

par(mar=rep(0,4))
plot(seattle)
points(x=data2[,4],y=data2[,5], cex=.5, pch=19, col="grey") #map our data points
points(x=mc[,2],y=mc[,3], pch=20, col=mypalette, cex=4)    #map our mean centers
legend(x="left",cex=1, fill=mypalette,bty="n",legend=mc[,1],
  title="Mean Centers with Std. Dist.")                                      #a basic legend
for(i in 1:length(sd[,1])){
  draw.circle(x=sd[i,2],y=sd[i,3],radius=sd[i,4],border=mypalette[i],lwd=3)
}

dev.off() 

################################################################################
#The next step in the ArcGIS lab draws standard ellipses for our data. The function
#is roughly the similar to the standard distances, but by allowing an ellipse we
#greatly complicate the function used to determine the smallest area shape that
#contains 68A% of the data points. There is a function that is supposed to do this
# in the aspace package, but I found it did not act as I expected it to. I also tried
# writing my own function again, but have not yet achieved something worth distributing
# for now, this section of the lab is unavailable in R.
################################################################################


#Global Clustering

################################################################################
#The first step when we are looking at any kind of clustering measure is to consider
# the spatial weights matrix that our calculations will be based on. In ArcGIS, we
# assign this matrix at the time we run the clustering function. In R, we calculate
# any number of matrices before we run our clustering function and simply include
# them as a parameter when we call the function.
#
## Contiguity Neighbors
Q1 <- poly2nb(seattle, queen=T)

#the functions we will use take in a listw object, so we use this conversion
#function. We set the options to note that we want basic row standardization
#and any polygons with zero neighbors will have that fact indicated in the list
#some functions generate errors when objects have zero neighbors, so we might
#want to generate an error if the function finds such a case.

Q1.listw<- nb2listw(Q1, style="W", zero.policy=TRUE)

## Plot the connections

par(mar=rep(0,4))
plot(Q1.listw,coords=coords,pch=19, cex=0.1, col="gray")
dev.off()

summary(Q1.listw)     #this gives us some good detail on our weights matrix

##Moran's I
# There are a number of ways to do the Moran's I test. The most straightforward is
moran.test(data$PctBlack, listw=Q1.listw, zero.policy=T)


################################################################################
#one of the additional functions we can do in R that is very useful is the 
# moran scatterplot. In lecture I talk about doing this function in GeoDa, but
# we can do something very similar here. The actual code is derived from a lab
# originally prepared by Paul Voss
#
#First we need to standardize our variable of interest
attach(data)
SPctBlack<-(PctBlack-mean(PctBlack))/sd(PctBlack)
hist(SPctBlack, nclass=50, probability=TRUE)
curve(dnorm(x, mean=0, sd=1), from=-3.1, to=3.1, add=T, col='red', lwd=2)

# Begin Paul Voss' MY.MORAN.PLOT function
my.moran.plot <- function (x, listw, zero.policy = FALSE, spChk = NULL,
    labels = NULL, xlab = NULL, ylab = NULL, quiet = FALSE, ...) 
{
    if (!inherits(listw, "listw")) 
        stop(paste(deparse(substitute(listw)), "is not a listw object"))
    xname <- deparse(substitute(x))
    if (!is.numeric(x)) 
        stop(paste(xname, "is not a numeric vector"))
    if (any(is.na(x))) 
        stop("NA in X")
    n <- length(listw$neighbours)
    if (n != length(x)) 
        stop("objects of different length")
    if (is.null(spChk)) 
        spChk <- get.spChkOption()
    if (spChk && !chkIDs(x, listw)) 
        stop("Check of data and weights ID integrity failed")
    labs <- TRUE
    if (is.logical(labels) && !labels) 
        labs <- FALSE
    if (is.null(labels) || length(labels) != n) 
        labels <- as.character(attr(listw, "region.id"))
    wx <- lag.listw(listw, x, zero.policy = zero.policy)
    if (is.null(xlab)) 
        xlab <- xname
    if (is.null(ylab)) 
        ylab <- paste("spatially lagged", xname)
    plot(x, wx, xlab = xlab, ylab = ylab, ...)
    if (zero.policy) {
        n0 <- wx == 0
        symbols(x[n0], wx[n0], inches = FALSE,
            circles = rep(diff(range(x))/50, 
            length(which(n0))), bg = "grey", add = TRUE)
    }
    xwx.lm <- lm(wx ~ x)
    abline(xwx.lm)
    abline(h = mean(wx), lty = 2)
    abline(v = mean(x), lty = 2)
    infl.xwx <- influence.measures(xwx.lm)
    is.inf <- which(apply(infl.xwx$is.inf, 1, any))
    points(x[is.inf], wx[is.inf], pch = 1, cex = 1) # CHANGED THIS
    if (labs) 
        text(x[is.inf], wx[is.inf], labels = labels[is.inf], 
            pos = 2, cex = 0.7)
    rownames(infl.xwx$infmat) <- labels
    if (!quiet) 
        summary(infl.xwx)
    invisible(infl.xwx)
}
#   End MY.MORAN.PLOT function
#
my.moran.plot(SPctBlack, Q1.listw, labels=FALSE,
	xlab=NULL, ylab=NULL, quiet=TRUE, pch=1)
#
# Unusually for one of these plots we don't have a lot of extreme outliers leveraging our statistic.
################################################################################

#The final step in our lab is to produce a Local Indicators of Spatial Autocorrelation
#Map, basically showing the most significant contributors to the scatterplot above
#in map form. Again, the code here is drawn from a lab originally prepared by 
#Paul Voss and modified minimally here.
    str(data)
#Begin by adding the variables we need to our data
detach()

data$SPctBlack<-(data$PctBlack-mean(data$PctBlack))/sd(data$PctBlack)
data$lagSPctBlack <- lag.listw(Q1.listw, data$SPctBlack)

#calculate the Local Moran's Value for each record
data$locm <- localmoran(data$PctBlack, Q1.listw, alternative="two.sided")

attach(data)

plot(SPctBlack, lagSPctBlack)
abline(h=0, v=0)             #should look very similar to Moran Scatterplot above

#  The trick to creating a LISA map is to define the categories by which we measure
# the significance of the local clustering. The code that follows is rather clunky
# but it selects only those records where R should plot a symbol indicating local clusters

data$hh <- (SPctBlack>= 0 & lagSPctBlack>= 0) & (locm[,5]<= 0.1)
data$ll <- (SPctBlack<= 0 & lagSPctBlack<= 0) & (locm[,5]<= 0.1)
data$hl <- (SPctBlack>= 0 & lagSPctBlack<= 0) & (locm[,5]<= 0.1)
data$lh <- (SPctBlack<= 0 & lagSPctBlack>= 0) & (locm[,5]<= 0.1)
data$ns <- locm[,5]> 0.1

# Create a single categorial variable summing up the
# five logical variables
data$var <- 0
data$var <- replace(data$var, (data$hh==TRUE), 1)
data$var <- replace(data$var, (data$ll==TRUE), 2)
data$var <- replace(data$var, (data$hl==TRUE), 3)
data$var <- replace(data$var, (data$lh==TRUE), 4)
data$var <- replace(data$var, (data$ns==TRUE), 5)

# Set the breaks for the thematic map classes
breaks <-seq(1,5,1)

# Set the corresponding labels for the thematic map classes
labels <- c("High-High", "Low-Low", "High-Low", "Low-High", "Not Signif.")

# Determine which map class each observation falls into based on
# the value of data$var
np <- findInterval(data$var, breaks, all.inside=FALSE)

# Assign colors to each map class--terrible colors, but they are "standard" for
# most LISA maps because they are the default in GeoDa
colors <- c("red", "blue", "lightpink", "skyblue2", "white")

# Finally! Plot the SpatialPolygonsDataFrame using
# specified breaks and color scheme

par(mar=rep(0,4))

plot(seattle)
plot(seattle, col=colors[np], add=TRUE)
mtext("LISA Map: Percent Black", cex=1.5, side=3, line=-1)
legend("topleft", legend=labels, fill=colors, bty="n")

dev.off()

##########################################
#As with the lab, we can repeat this process using the LogBlack variable to deal
# with the fact that our main varibale is not normally distributed
detach()

data$SLogBlack<-(data$LogBlack-mean(data$LogBlack))/sd(data$LogBlack)
data$lagSLogBlack <- lag.listw(Q1.listw, data$SLogBlack)

#calculate the Local Moran's Value for each record
data$locm <- localmoran(data$LogBlack, Q1.listw, alternative="two.sided")

attach(data)

data$hh <- (SLogBlack>= 0 & lagSLogBlack>= 0) & (locm[,5]<= 0.1)
data$ll <- (SLogBlack<= 0 & lagSLogBlack<= 0) & (locm[,5]<= 0.1)
data$hl <- (SLogBlack>= 0 & lagSLogBlack<= 0) & (locm[,5]<= 0.1)
data$lh <- (SLogBlack<= 0 & lagSLogBlack>= 0) & (locm[,5]<= 0.1)
data$ns <- locm[,5]> 0.1

# Create a single categorial variable summing up the
# five logical variables
data$var <- 0
data$var <- replace(data$var, (data$hh==TRUE), 1)
data$var <- replace(data$var, (data$ll==TRUE), 2)
data$var <- replace(data$var, (data$hl==TRUE), 3)
data$var <- replace(data$var, (data$lh==TRUE), 4)
data$var <- replace(data$var, (data$ns==TRUE), 5)

# Set the breaks for the thematic map classes
breaks <-seq(1,5,1)

# Set the corresponding labels for the thematic map classes
labels <- c("High-High", "Low-Low", "High-Low", "Low-High", "Not Signif.")

# Determine which map class each observation falls into based on
# the value of data$var
np <- findInterval(data$var, breaks, all.inside=FALSE)

# Assign colors to each map class--terrible colors, but they are "standard" for
# most LISA maps because they are the default in GeoDa
colors <- c("red", "blue", "lightpink", "skyblue2", "white")

# Finally! Plot the SpatialPolygonsDataFrame using
# specified breaks and color scheme

par(mar=rep(0,4))

plot(seattle)
plot(seattle, col=colors[np], add=TRUE)
mtext("LISA Map: Log Percent Black", cex=1.5, side=3, line=-1)
legend("topleft", legend=labels, fill=colors, bty="n")

dev.off()

# End Lab